@@ -25,15 +25,23 @@
 
 #include <config.h>
 
+#ifndef GNOME_DISABLE_DEPRECATED_SOURCE
+
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
 #include <glib/gi18n-lib.h>
+
+#undef GTK_DISABLE_DEPRECATED /* for GtkOptionMenu */
 #include <gtk/gtk.h>
 
 #include <libgnome/gnome-macros.h>
 
 #include <libgnomeui/gnome-uidefs.h>
+
+#include <libgnomevfs/gnome-vfs-utils.h>
+
+#undef GNOME_DISABLE_DEPRECATED
 #include <libgnomeui/gnome-icon-entry.h>
 
 #include <libgnome/gnome-desktop-item.h>
@@ -80,7 +88,6 @@ struct _GnomeDItemEditPrivate {
 };
 
 static void gnome_ditem_edit_class_init   (GnomeDItemEditClass *klass);
-static void gnome_ditem_edit_instance_init(GnomeDItemEdit      *dee);
 
 static void gnome_ditem_edit_destroy      (GtkObject           *object);
 static void gnome_ditem_edit_finalize     (GObject             *object);
@@ -92,10 +99,6 @@ static void gnome_ditem_edit_changed      (GnomeDItemEdit      *dee);
 static void gnome_ditem_edit_icon_changed (GnomeDItemEdit      *dee);
 static void gnome_ditem_edit_name_changed (GnomeDItemEdit      *dee);
 
-static void destroy_tooltip (GtkObject *object);
-static void set_tooltip (GnomeDItemEdit *dee, GtkWidget *widget,
-			 const gchar *description);
-
 enum {
         CHANGED,
         ICON_CHANGED,
@@ -107,10 +110,7 @@ static gint ditem_edit_signals[LAST_SIGNAL] = { 0 };
 
 /* The following defines the get_type */
 
-GNOME_CLASS_BOILERPLATE (GnomeDItemEdit, gnome_ditem_edit,
-			 GtkNotebook, GTK_TYPE_NOTEBOOK)
-
-
+G_DEFINE_TYPE(GnomeDItemEdit, gnome_ditem_edit, GTK_TYPE_NOTEBOOK)
 
 static void
 gnome_ditem_edit_class_init (GnomeDItemEditClass *klass)
@@ -674,6 +674,9 @@ setup_translations_list (GnomeDItemEdit *dee)
 					"text", 3, NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
 
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model),
+					      0, GTK_SORT_ASCENDING);
+
 	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (tree));
 
 	gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (tree), FALSE);
@@ -752,22 +755,19 @@ make_advanced_page (GnomeDItemEdit *dee)
 	gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 0);
 	gtk_widget_set_size_request (entry, 30, -1);
 	dee->_priv->transl_lang_entry = entry;
-
-	set_tooltip (dee, GTK_WIDGET(entry), _("Language"));
+	gtk_widget_set_tooltip_text (entry, _("Language"));
 
 	entry = gtk_entry_new ();
 	gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 0);
 	gtk_widget_set_size_request (entry, 80, -1);
 	dee->_priv->transl_name_entry = entry;
-
-	set_tooltip (dee, GTK_WIDGET(entry), _("Name"));
+	gtk_widget_set_tooltip_text (entry, _("Name"));
 
 	entry = gtk_entry_new ();
 	gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 0);
 	gtk_widget_set_size_request (entry, 80, -1);
 	dee->_priv->transl_generic_name_entry = entry;
-
-	set_tooltip (dee, GTK_WIDGET (entry), _("Generic name"));
+	gtk_widget_set_tooltip_text (entry, _("Generic name"));
 
 	/* FIXME: transl_icon_entry, locale specific icons */
 
@@ -775,29 +775,27 @@ make_advanced_page (GnomeDItemEdit *dee)
 	gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 0);
 	gtk_widget_set_size_request (entry, 80, -1);
 	dee->_priv->transl_comment_entry = entry;
-
-	set_tooltip (dee, GTK_WIDGET(entry), _("Comment"));
+	gtk_widget_set_tooltip_text (entry, _("Comment"));
 
 	button = gtk_button_new_with_mnemonic (_("_Add/Set"));
 	gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (translations_add), dee);
+	gtk_widget_set_tooltip_text (button,
+				     _("Add or Set Name/Comment Translations"));
   
-	set_tooltip (dee, GTK_WIDGET(button),
-		    _("Add or Set Name/Comment Translations"));
-
 	button = gtk_button_new_with_mnemonic (_("Re_move"));
 	gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (translations_remove), dee);
-	set_tooltip (dee, GTK_WIDGET(button),
-		    _("Remove Name/Comment Translation"));
+	gtk_widget_set_tooltip_text (button,
+				     _("Remove Name/Comment Translation"));
   
 	return vbox;
 }
 
 static void
-gnome_ditem_edit_instance_init (GnomeDItemEdit *dee)
+gnome_ditem_edit_init (GnomeDItemEdit *dee)
 {
 	GtkWidget *page;
 
@@ -829,6 +827,8 @@ gnome_ditem_edit_instance_init (GnomeDItemEdit *dee)
  * for the purpose of editing #GnomeDesktopItems
  *
  * Returns: Newly-created #GnomeDItemEdit widget.
+ *
+ * Deprecated: 2.18:
  */
 GtkWidget *
 gnome_ditem_edit_new (void)
@@ -856,9 +856,7 @@ gnome_ditem_edit_destroy (GtkObject *object)
 		gnome_desktop_item_unref (de->_priv->ditem);
 	de->_priv->ditem = NULL; /* just for sanity */
 
-	destroy_tooltip (object);
-
-	GNOME_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+	GTK_OBJECT_CLASS (gnome_ditem_edit_parent_class)->destroy (object);
 }
 
 static void
@@ -874,7 +872,7 @@ gnome_ditem_edit_finalize (GObject *object)
 	g_free(de->_priv);
 	de->_priv = NULL;
 
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
+	G_OBJECT_CLASS (gnome_ditem_edit_parent_class)->finalize (object);
 }
 
 /* set sensitive for directory/other type of a ditem */
@@ -1044,6 +1042,19 @@ get_language (void)
 	return NULL;
 }
 
+static void
+ensure_item_localefiled (GnomeDesktopItem *ditem,
+			 const char       *field)
+{
+	const char *localized;
+
+	if (gnome_desktop_item_get_string (ditem, field) == NULL) {
+		localized = gnome_desktop_item_get_localestring (ditem, field);
+		if (localized != NULL)
+			gnome_desktop_item_set_string (ditem, field, localized);
+	}
+}
+
 static void
 gnome_ditem_edit_sync_ditem (GnomeDItemEdit *dee)
 {
@@ -1053,6 +1064,7 @@ gnome_ditem_edit_sync_ditem (GnomeDItemEdit *dee)
 	GtkWidget        *entry;
 	const char       *type;
 	const char       *uri;
+	char             *free_uri;
 	const char       *attr;
 	char             *file;
 	gboolean          ret;
@@ -1073,12 +1085,18 @@ gnome_ditem_edit_sync_ditem (GnomeDItemEdit *dee)
 	gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_TYPE, type);
 
 	/* hack really */
-	if (type && !strcmp (type, "Link"))
+	free_uri = NULL;
+	if (type && !strcmp (type, "Link")) {
+		free_uri = gnome_vfs_make_uri_canonical (uri);
+		uri = free_uri;
+
 		attr = GNOME_DESKTOP_ITEM_URL;
-	else
+	} else
 		attr  = GNOME_DESKTOP_ITEM_EXEC;
 
 	gnome_desktop_item_set_string (ditem, attr, uri);
+	if (free_uri)
+		g_free (free_uri);
 
 	gnome_desktop_item_set_string (
 		ditem, GNOME_DESKTOP_ITEM_TRY_EXEC,
@@ -1165,6 +1183,14 @@ gnome_ditem_edit_sync_ditem (GnomeDItemEdit *dee)
 	gnome_desktop_item_set_localestring (
 			ditem, GNOME_DESKTOP_ITEM_COMMENT,
 			gtk_entry_get_text (GTK_ENTRY(dee->_priv->comment_entry)));
+
+	/* Make sure we set the "C" locale strings to the terms we set here.
+	 * This is so that if the user logs into another locale they get their
+	 * own description there rather then empty. It is not the C locale
+	 * however, but the user created this entry herself so it's OK */
+	ensure_item_localefiled (ditem, GNOME_DESKTOP_ITEM_NAME);
+	ensure_item_localefiled (ditem, GNOME_DESKTOP_ITEM_GENERIC_NAME);
+	ensure_item_localefiled (ditem, GNOME_DESKTOP_ITEM_COMMENT);
 			
 	dee->_priv->ui_dirty = FALSE;
 }
@@ -1173,11 +1199,14 @@ gnome_ditem_edit_sync_ditem (GnomeDItemEdit *dee)
  * gnome_ditem_edit_load_uri
  * @dee: #GnomeDItemEdit object to work with
  * @uri: file to load into the editting areas
+ * @error: location to store error, or NULL
  *
  * Description: Load a .desktop file and update the editting areas
  * of the object accordingly.
  *
  * Returns:  %TRUE if successful, %FALSE otherwise
+ *
+ * Deprecated: 2.18:
  */
 gboolean
 gnome_ditem_edit_load_uri (GnomeDItemEdit *dee,
@@ -1216,7 +1245,7 @@ gnome_ditem_edit_load_uri (GnomeDItemEdit *dee,
  * Note that since the entire item is stored, any hidden fields will be
  * preserved when you later get it with #gnome_ditem_edit_get_ditem.
  *
- * Returns:
+ * Deprecated: 2.18:
  */
 void
 gnome_ditem_edit_set_ditem (GnomeDItemEdit *dee,
@@ -1245,6 +1274,8 @@ gnome_ditem_edit_set_ditem (GnomeDItemEdit *dee,
  * make a copy of it with #gnome_desktop_item_copy
  *
  * Returns: a pointer to the internal #GnomeDesktopItem structure.
+ *
+ * Deprecated: 2.18:
  */
 GnomeDesktopItem *
 gnome_ditem_edit_get_ditem (GnomeDItemEdit *dee)
@@ -1270,7 +1301,7 @@ gnome_ditem_edit_get_ditem (GnomeDItemEdit *dee)
  * Description: Clear the editting areas.  And unref any
  * stored #GnomeDesktopItem.
  *
- * Returns:
+ * Deprecated: 2.18:
  */
 void
 gnome_ditem_edit_clear (GnomeDItemEdit *dee)
@@ -1287,7 +1318,7 @@ gnome_ditem_edit_clear (GnomeDItemEdit *dee)
         gtk_entry_set_text(GTK_ENTRY(dee->_priv->name_entry), "");
         gtk_entry_set_text(GTK_ENTRY(dee->_priv->generic_name_entry), "");
         gtk_entry_set_text(GTK_ENTRY(dee->_priv->comment_entry),"");
-        gtk_entry_set_text(GTK_ENTRY(dee->_priv->exec_entry), "");  
+        gnome_file_entry_set_filename (GNOME_FILE_ENTRY (dee->_priv->exec_entry), "");
         gtk_entry_set_text(GTK_ENTRY(dee->_priv->tryexec_entry), "");
         gtk_entry_set_text(GTK_ENTRY(dee->_priv->doc_entry), "");
         gnome_icon_entry_set_filename (GNOME_ICON_ENTRY (dee->_priv->icon_entry), "");
@@ -1332,6 +1363,8 @@ gnome_ditem_edit_name_changed (GnomeDItemEdit *dee)
  * Description: Get the icon filename.
  *
  * Returns: a newly allocated string with the filename of the icon
+ *
+ * Deprecated: 2.18:
  */
 gchar *
 gnome_ditem_edit_get_icon (GnomeDItemEdit *dee)
@@ -1349,6 +1382,8 @@ gnome_ditem_edit_get_icon (GnomeDItemEdit *dee)
  * Description: Get the Name field from the ditem for the current language.
  *
  * Returns: a newly allocated string with the name of the ditem
+ *
+ * Deprecated: 2.18:
  */
 gchar *
 gnome_ditem_edit_get_name (GnomeDItemEdit *dee)
@@ -1363,6 +1398,14 @@ gnome_ditem_edit_get_name (GnomeDItemEdit *dee)
 }
 
 /* eeeeeeeeek!, evil api */
+/**
+ * gnome_ditem_edit_grab_focus
+ * @dee: #GnomeDItemEdit object to work with
+ *
+ * Description: Give the focus to the editor.
+ *
+ * Deprecated: 2.18:
+ */
 void
 gnome_ditem_edit_grab_focus (GnomeDItemEdit *dee)
 {
@@ -1371,6 +1414,16 @@ gnome_ditem_edit_grab_focus (GnomeDItemEdit *dee)
 
 	gtk_widget_grab_focus (dee->_priv->name_entry);
 }
+
+/**
+ * gnome_ditem_edit_set_editable
+ * @dee: #GnomeDItemEdit object to work with
+ * @editable: %TRUE to set the object editable
+ *
+ * Description: Set the object editable.
+ *
+ * Deprecated: 2.18:
+ */
 void
 gnome_ditem_edit_set_editable (GnomeDItemEdit *dee,
 			       gboolean editable)
@@ -1383,7 +1436,15 @@ gnome_ditem_edit_set_editable (GnomeDItemEdit *dee,
 	gtk_widget_set_sensitive (dee->_priv->child2, editable);
 }
 
-/* set the string type of the entry */
+/**
+ * gnome_ditem_edit_set_entry_type
+ * @dee: #GnomeDItemEdit object to work with
+ * @type: a type
+ *
+ * Description: Set the string type of the entry.
+ *
+ * Deprecated: 2.18:
+ */
 void
 gnome_ditem_edit_set_entry_type (GnomeDItemEdit *dee,
 				 const char     *type)
@@ -1410,7 +1471,15 @@ gnome_ditem_edit_set_entry_type (GnomeDItemEdit *dee,
 	}
 }
 
-/* force directory only */
+/**
+ * gnome_ditem_edit_set_entry_type
+ * @dee: #GnomeDItemEdit object to work with
+ * @directory_only: %TRUE to force directory only
+ *
+ * Description: Force directory only.
+ *
+ * Deprecated: 2.18:
+ */
 void
 gnome_ditem_edit_set_directory_only (GnomeDItemEdit *dee,
 				     gboolean        directory_only)
@@ -1445,38 +1514,4 @@ gnome_ditem_edit_set_directory_only (GnomeDItemEdit *dee,
 	}
 }
 
-static void
-destroy_tooltip (GtkObject *object)
-{
-	GtkTooltips *tooltips;
-
-	tooltips = g_object_get_data (G_OBJECT (object), "tooltips");
-	if (tooltips) {
-		g_object_unref (tooltips);
-		g_object_set_data (G_OBJECT (object), "tooltips", NULL);
-	}
-}
-
-/**
- * set_tooltip
- * Description : Set @description as the tooltip for the @widget.
- *		 Creates the GtkTooltips structure if not already present.
- */
-static void
-set_tooltip (GnomeDItemEdit *dee, GtkWidget *widget, const gchar *description)
-{
-        GtkTooltips *tooltips;
-
-	tooltips = g_object_get_data (G_OBJECT (dee), "tooltips");
-
-	/* create if not already present */
-	if (!tooltips) {
-		tooltips = gtk_tooltips_new ();
-		g_return_if_fail (tooltips != NULL);
-		g_object_ref (tooltips);
-		gtk_object_sink (GTK_OBJECT (tooltips));
-		g_object_set_data (G_OBJECT (dee), "tooltips", tooltips);
-	}
-
-        gtk_tooltips_set_tip (tooltips, widget, description, NULL);
-}
+#endif /* GNOME_DISABLE_DEPRECATED_SOURCE */
